@@ -1,11 +1,11 @@
 /* A replacement function, for systems that lack strndup.
 
-   Copyright (C) 1996, 1997, 1998, 2001, 2002, 2003, 2005, 2006 Free
-   Software Foundation, Inc.
+   Copyright (C) 1996, 1997, 1998, 2001, 2002, 2003, 2005, 2006, 2007
+   Free Software Foundation, Inc.
 
    This program is free software; you can redistribute it and/or modify it
    under the terms of the GNU Lesser General Public License as published by the
-   Free Software Foundation; either version 2.1, or (at your option) any
+   Free Software Foundation; either version 2, or (at your option) any
    later version.
 
    This program is distributed in the hope that it will be useful,
@@ -18,12 +18,10 @@
    Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.  */
 
 #include <config.h>
-#include "strndup.h"
 
-#include <stdlib.h>
 #include <string.h>
 
-#include "strnlen.h"
+#include <stdlib.h>
 
 char *
 strndup (char const *s, size_t n)
