@@ -26,9 +26,7 @@
 #include <stdlib.h>
 #include <stdio.h>
 #include <string.h>
-#include <inttypes.h>
 #include <sys/types.h>
-#include <time.h>
 
 #include <libprelude/prelude-log.h>
 #include <libprelude/idmef.h>
