@@ -26,10 +26,8 @@
 
 #include "config.h"
 
-#include "snprintf.h"
-#include "strdup.h"
-#include "strndup.h"
-#include "time_r.h"
-#include "vsnprintf.h"
+#include <string.h>
+#include <stdio.h>
+#include <time.h>
 
 #endif
